"use strict";
/**
 * Components - Navigation
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Navigation = void 0;
const utils_1 = require("../../utils/utils");
/**
 * Class - recursively generate navigation output
 */
class Navigation {
    /**
     * Store all navigations
     *
     * @type {import('./NavigationTypes').Navigations[]}
     */
    navigations = [];
    /**
     * Store all navigation items
     *
     * @type {import('./NavigationTypes').NavigationItem[]}
     */
    items = [];
    /**
     * Current link to compare against
     *
     * @type {string}
     */
    current = '';
    /**
     * Store initialize success
     *
     * @type {boolean}
     */
    init = false;
    /**
     * Store navigation items by id
     *
     * @private
     * @type {import('./NavigationTypes').NavigationItemsById}
     */
    #itemsById = {};
    /**
     * Store navigations by location
     *
     * @private
     * @type {import('./NavigationTypes').NavigationByLocation}
     */
    #navigationsByLocation = {};
    /**
     * Set properties and initialize
     *
     * @param {import('./NavigationTypes').NavigationProps} props
     */
    constructor(props) {
        this.init = this.#initialize(props);
    }
    /**
     * Initialize - check required props and set props
     *
     * @private
     * @param {import('./NavigationTypes').NavigationProps} props
     * @return {boolean}
     */
    #initialize(props) {
        /* Check props is object */
        if (!(0, utils_1.isObjectStrict)(props)) {
            return false;
        }
        /* Defaults */
        const { navigations = [], items = [], current = '' } = props;
        /* Check that required items exist */
        if (!(0, utils_1.isArrayStrict)(navigations) || !(0, utils_1.isArrayStrict)(items)) {
            return false;
        }
        /* Set variables */
        this.navigations = navigations;
        this.items = items;
        this.current = current;
        /* Items by id */
        this.items.forEach(item => {
            const info = this.#getItemInfo(item);
            if (info !== undefined && (0, utils_1.isStringStrict)(info.id)) {
                this.#itemsById[info.id] = info;
            }
        });
        /* Navigations by location */
        this.navigations.forEach(nav => {
            const fields = utils_1.getProp.self(nav);
            if (!(0, utils_1.isObjectStrict)(fields)) {
                return;
            }
            const { title = '', location = '', items = [] } = fields;
            if ((0, utils_1.isStringStrict)(title) && (0, utils_1.isStringStrict)(location) && (0, utils_1.isArrayStrict)(items)) {
                this.#navigationsByLocation[location.toLowerCase().replace(/\s+/g, '')] = {
                    title,
                    items
                };
            }
        });
        /* Init successful */
        return true;
    }
    /**
     * Normalize navigation item props
     *
     * @private
     * @param {import('./NavigationTypes').NavigationItem} item
     * @return {import('./NavigationTypes').NavigationItem|undefined}
     */
    #getItemInfo(item) {
        const fields = utils_1.getProp.self(item);
        if (!(0, utils_1.isObjectStrict)(fields)) {
            return;
        }
        const { title = '', internalLink, externalLink = '', children } = fields;
        let id = (0, utils_1.isStringStrict)(title) ? title : '';
        let external = false;
        if ((0, utils_1.isStringStrict)(externalLink)) {
            id = externalLink;
            external = true;
        }
        const internalId = utils_1.getProp.id(internalLink);
        if ((0, utils_1.isStringStrict)(internalId)) {
            id = internalId;
        }
        const link = (0, utils_1.getLink)(internalLink, externalLink);
        const props = {
            id,
            title,
            link,
            external
        };
        if ((0, utils_1.isStringStrict)(link) && !external) {
            props.current = props.link === this.current;
        }
        let descendentCurrent = false;
        if ((0, utils_1.isArrayStrict)(children)) {
            const c = [];
            descendentCurrent = this.#recurseItemChildren(children, c);
            props.children = c;
        }
        if (descendentCurrent) {
            props.descendentCurrent = descendentCurrent;
        }
        Object.keys(fields).forEach((f) => {
            if (props[f] === undefined) {
                props[f] = fields[f];
            }
        });
        return props;
    }
    /**
     * Loop through items to check and set children
     *
     * @private
     * @param {import('./NavigationTypes').NavigationItem[]} children
     * @param {import('./NavigationTypes').NavigationItem[]} store
     * @return {boolean}
     */
    #recurseItemChildren(children = [], store = []) {
        let childCurrent = false;
        children.forEach(child => {
            const info = this.#getItemInfo(child);
            if (info === undefined) {
                return;
            }
            const { current = false } = info;
            if (current) {
                childCurrent = true;
            }
            store.push(info);
        });
        return childCurrent;
    }
    /**
     * Return navigation items by id
     *
     * @private
     * @param {import('./NavigationTypes').NavigationItem[]} items
     * @return {import('./NavigationTypes').NavigationItem[]}
     */
    #getItems(items = []) {
        if (items.length === 0) {
            return [];
        }
        const resItems = [];
        items.forEach(item => {
            const fields = utils_1.getProp.self(item);
            if (!(0, utils_1.isObjectStrict)(fields)) {
                return;
            }
            const { title = '', internalLink, externalLink } = fields;
            let id = '';
            if ((0, utils_1.isStringStrict)(title)) {
                id = title;
            }
            if ((0, utils_1.isStringStrict)(externalLink)) {
                id = externalLink;
            }
            const internalId = utils_1.getProp.id(internalLink);
            if ((0, utils_1.isStringStrict)(internalId)) {
                id = internalId;
            }
            const storedItem = this.#itemsById[id];
            if (storedItem !== undefined) {
                resItems.push(storedItem);
            }
        });
        return resItems;
    }
    /**
     * Loop through items to create html
     *
     * @private
     * @param {import('./NavigationTypes').NavigationItem[]} items
     * @param {import('../../global/globalTypes').HtmlString} output
     * @param {import('./NavigationTypes').NavigationOutputArgs} args
     * @param {number} depth
     * @param {number} maxDepth
     * @return {void}
     */
    #recurseOutput = (items = [], output, args, depth = -1, maxDepth) => {
        depth += 1;
        if (maxDepth !== undefined && depth > maxDepth) {
            return;
        }
        const listFilterArgs = { args, output, items, depth };
        if ((0, utils_1.isFunction)(args.filterBeforeList)) {
            args.filterBeforeList(listFilterArgs);
        }
        const listClasses = (0, utils_1.isStringStrict)(args.listClass) ? ` class="${args.listClass}"` : '';
        const listAttrs = (0, utils_1.isStringStrict)(args.listAttr) ? ` ${args.listAttr}` : '';
        output.html += `<ul data-depth="${depth}"${listClasses}${listAttrs}>`;
        items.forEach((item, index) => {
            const { title = '', link = '', external = false, children = [], current = false, descendentCurrent = false } = item;
            /* Filters args */
            const filterArgs = { args, item, output, index, items, depth };
            /* Item start */
            if ((0, utils_1.isFunction)(args.filterBeforeItem)) {
                args.filterBeforeItem(filterArgs);
            }
            const itemClasses = (0, utils_1.isStringStrict)(args.itemClass) ? ` class="${args.itemClass}"` : '';
            let itemAttrs = (0, utils_1.isStringStrict)(args.itemAttr) ? ` ${args.itemAttr}` : '';
            if (current) {
                itemAttrs += ' data-current="true"';
            }
            if (descendentCurrent) {
                itemAttrs += ' data-descendent-current="true"';
            }
            output.html += `<li data-depth="${depth}"${itemClasses}${itemAttrs}>`;
            /* Link start */
            if ((0, utils_1.isFunction)(args.filterBeforeLink)) {
                args.filterBeforeLink(filterArgs);
            }
            const linkClassesArr = [];
            if ((0, utils_1.isStringStrict)(args.linkClass)) {
                linkClassesArr.push(args.linkClass);
            }
            if (!external && (0, utils_1.isStringStrict)(args.internalLinkClass)) {
                linkClassesArr.push(args.internalLinkClass);
            }
            const linkClasses = (linkClassesArr.length > 0) ? ` class="${linkClassesArr.join(' ')}"` : '';
            const linkAttrsArr = [link !== '' ? `href="${link}"` : 'type="button"'];
            if ((0, utils_1.isStringStrict)(args.linkAttr)) {
                linkAttrsArr.push(args.linkAttr);
            }
            if (current) {
                linkAttrsArr.push('data-current="true"');
                if (link !== '') {
                    linkAttrsArr.push('aria-current="page"');
                }
            }
            if (descendentCurrent) {
                linkAttrsArr.push('data-descendent-current="true"');
            }
            const linkAttrs = (linkAttrsArr.length > 0) ? ` ${linkAttrsArr.join(' ')}` : '';
            const linkTag = link !== '' ? 'a' : 'button';
            output.html += `<${linkTag} data-depth="${depth}"${linkClasses}${linkAttrs}>`;
            if ((0, utils_1.isFunction)(args.filterBeforeLinkText)) {
                args.filterBeforeLinkText(filterArgs);
            }
            output.html += title;
            if ((0, utils_1.isFunction)(args.filterAfterLinkText)) {
                args.filterAfterLinkText(filterArgs);
            }
            /* Link end */
            output.html += `</${linkTag}>`;
            if ((0, utils_1.isFunction)(args.filterAfterLink)) {
                args.filterAfterLink(filterArgs);
            }
            /* Nested content */
            if (children.length > 0) {
                this.#recurseOutput(children, output, args, depth, maxDepth);
            }
            /* Item end */
            output.html += '</li>';
            if ((0, utils_1.isFunction)(args.filterAfterItem)) {
                args.filterAfterItem(filterArgs);
            }
        });
        output.html += '</ul>';
        if ((0, utils_1.isFunction)(args.filterAfterList)) {
            args.filterAfterList(listFilterArgs);
        }
    };
    /**
     * Return navigation html output
     *
     * @param {string} location
     * @param {import('./NavigationTypes').NavigationOutputArgs} args
     * @param {number} maxDepth
     * @return {string} HTML - ul
     */
    getOutput(location = '', args, maxDepth) {
        if (this.#navigationsByLocation[location] === undefined) {
            return '';
        }
        const items = this.#navigationsByLocation[location].items;
        const normalizedItems = this.#getItems(items);
        args = Object.assign({
            listClass: '',
            listAttr: '',
            itemClass: '',
            itemAttr: '',
            linkClass: '',
            internalLinkClass: '',
            linkAttr: '',
            filterBeforeList: () => { },
            filterAfterList: () => { },
            filterBeforeItem: () => { },
            filterAfterItem: () => { },
            filterBeforeLink: () => { },
            filterAfterLink: () => { },
            filterBeforeLinkText: () => { },
            filterAfterLinkText: () => { }
        }, args);
        const output = { html: '' };
        this.#recurseOutput(normalizedItems, output, args, -1, maxDepth);
        return output.html;
    }
    /**
     * Return breadcrumbs html output
     *
     * @param {import('./NavigationTypes').NavigationBreadcrumbItem[]} items
     * @param {import('./NavigationTypes').NavigationBreadcrumbOutputArgs} args
     * @param {string} current
     * @return {string} HTML - ol
     */
    getBreadcrumbs(items = [], args, current = '') {
        /* Items required */
        if (!(0, utils_1.isArrayStrict)(items)) {
            return '';
        }
        /* Args defaults */
        args = Object.assign({
            listClass: '',
            listAttr: '',
            itemClass: '',
            itemAttr: '',
            linkClass: '',
            internalLinkClass: '',
            linkAttr: '',
            currentClass: '',
            a11yClass: 'a-visually-hidden',
            filterBeforeLink: () => { },
            filterAfterLink: () => { }
        }, args);
        /* List attributes */
        const listClasses = (0, utils_1.isStringStrict)(args.listClass) ? ` class="${args.listClass}"` : '';
        const listAttrs = (0, utils_1.isStringStrict)(args.listAttr) ? ` ${args.listAttr}` : '';
        /* Loop through items */
        const itemClasses = (0, utils_1.isStringStrict)(args.itemClass) ? ` class="${args.itemClass}"` : '';
        const itemAttrs = (0, utils_1.isStringStrict)(args.itemAttr) ? ` ${args.itemAttr}` : '';
        const lastItemIndex = items.length - 1;
        const itemsArr = items.map((item, index) => {
            const { title } = item;
            /* Title required */
            if (!(0, utils_1.isStringStrict)(title)) {
                return '';
            }
            /* Permalink required */
            const slug = (0, utils_1.getSlug)({
                id: item.id,
                slug: item.slug,
                contentType: item.contentType,
                linkContentType: item.linkContentType
            });
            const permalink = (0, utils_1.isString)(slug) ? (0, utils_1.getPermalink)(slug) : '';
            if (permalink === '') {
                return '';
            }
            /* Output store */
            const output = { html: '' };
            /* Check if last */
            const isLastLevel = lastItemIndex === index;
            /* Filter args */
            const filterArgs = { output, isLastLevel };
            /* Item */
            output.html += `<li${itemClasses}${itemAttrs} data-last-level="${isLastLevel.toString()}">`;
            /* Link */
            if ((0, utils_1.isFunction)(args.filterBeforeLink)) {
                args.filterBeforeLink(filterArgs);
            }
            const linkClassesArr = [];
            if ((0, utils_1.isStringStrict)(args.linkClass)) {
                linkClassesArr.push(args.linkClass);
            }
            if ((0, utils_1.isStringStrict)(args.internalLinkClass)) {
                linkClassesArr.push(args.internalLinkClass);
            }
            const linkClasses = (linkClassesArr.length > 0) ? ` class="${linkClassesArr.join(' ')}"` : '';
            const linkAttrs = (0, utils_1.isStringStrict)(args.linkAttr) ? ` ${args.linkAttr}` : '';
            output.html += `<a${linkClasses} href="${permalink}"${linkAttrs}>${title}</a>`;
            if ((0, utils_1.isFunction)(args.filterAfterLink)) {
                args.filterAfterLink(filterArgs);
            }
            /* Close item */
            output.html += '</li>';
            return output.html;
        });
        /* Output */
        const currentClasses = (0, utils_1.isStringStrict)(args.currentClass) ? ` class="${args.currentClass}"` : '';
        const a11yClasses = (0, utils_1.isStringStrict)(args.a11yClass) ? ` class="${args.a11yClass}"` : '';
        return `
      <ol${listClasses}${listAttrs}>
        ${itemsArr.join('')}
        <li${itemClasses}${itemAttrs} data-current="true">
          <span${currentClasses}>${current}<span${a11yClasses}> (current page)</span></span>
        </li>
      </ol>
    `;
    }
}
exports.Navigation = Navigation;
