"use strict";
/**
 * Utils - Get Contentful Data
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getContentfulData = void 0;
const contentful_resolve_response_1 = __importDefault(require("contentful-resolve-response"));
const utils_1 = require("../utils");
const config_1 = require("../../config/config");
/**
 * Function - fetch data from contentful cms or cache
 *
 * @param {string} key
 * @param {import('./getContentfulDataTypes').ContentfulDataParams} params
 * @return {Promise<import('./getContentfulDataTypes').ContentfulDataReturn>}
 */
const getContentfulData = async (key = '', params = {}) => {
    try {
        /* Key required for cache */
        if (!(0, utils_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_1.config.env.cache) {
            let cacheData = {};
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, utils_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, utils_1.isObject)(cacheData) && Object.keys(cacheData).length > 0) {
                return structuredClone(cacheData);
            }
        }
        /* Credentials */
        const { space, previewAccessToken, deliveryAccessToken, previewHost, deliveryHost } = config_1.config.cms;
        let accessToken = previewAccessToken;
        let host = previewHost;
        if (config_1.config.env.prod) {
            accessToken = deliveryAccessToken;
            host = deliveryHost;
        }
        if (!(0, utils_1.isStringStrict)(space) || !(0, utils_1.isStringStrict)(accessToken) || !(0, utils_1.isStringStrict)(host)) {
            throw new Error('No credentials');
        }
        /* Params */
        let url = `https://${host}/spaces/${space}/environments/master/entries?access_token=${accessToken}`;
        Object.keys(params).forEach(p => {
            url += `&${p}=${params[p].toString()}`;
        });
        /* New data */
        const resp = await fetch(url);
        const data = await resp.json();
        if (!(0, utils_1.isObjectStrict)(data)) {
            throw new Error('No data');
        }
        if (data.items === undefined) {
            throw new Error('No items');
        }
        data.items = (0, contentful_resolve_response_1.default)(data);
        /* Store in cache */
        if (config_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, utils_1.applyFilters)('cacheData', data, cacheDataFilterArgs);
        }
        /* Output */
        return data;
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error fetching Contentful data: ', error);
        return {};
    }
};
exports.getContentfulData = getContentfulData;
